@@ -1,4 +1,5 @@
-// Copyright (c) 2002 Rob Kaper <cap@capsi.com>
+// Copyright © 2002 Rob Kaper <cap@capsi.com>
+//             2010-2015 Sylvain Rochet <gradator@gradator.net>
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License
@@ -10,9 +11,11 @@
 // General Public License for more details.
 //
 // You should have received a copy of the GNU General Public License
-// along with this program; see the file COPYING.  If not, write to
-// the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-// Boston, MA 02111-1307, USA.
+// along with this program; see the file COPYING. If not, see
+// <http://www.gnu.org/licenses/>.
+
+#include <stdio.h>
+#include <stdarg.h>
 
 #include "display.h"
 #include "estate.h"
@@ -26,6 +29,12 @@ DisplayButton::DisplayButton(const std::string command, const std::string captio
 	m_enabled = enabled;
 }
 
+DisplayButton::DisplayButton(const DisplayButton *button) {
+	m_command = button->m_command;
+	m_caption = button->m_caption;
+	m_enabled = button->m_enabled;
+}
+
 const std::string DisplayButton::command()
 {
 	return m_command;
@@ -43,14 +52,28 @@ bool DisplayButton::enabled()
 
 Display::Display()
 {
-	m_estate = 0;
+	m_estate = NULL;
 	m_text = "";
-	m_clearText = m_clearButtons = false;
+	m_clearText = m_clearButtons = m_clearEstate = false;
+}
+
+Display::Display(const Display *display) {
+	m_estate = display->m_estate;
+	m_text = display->m_text;
+	m_clearText = display->m_clearText;
+	m_clearButtons = display->m_clearButtons;
+	m_clearEstate = display->m_clearEstate;
+
+	std::vector<DisplayButton *> buttons = display->m_buttons;
+	for (std::vector<DisplayButton *>::iterator it = buttons.begin() ; it != buttons.end(); ++it) {
+		DisplayButton *button = new DisplayButton(*it);
+		m_buttons.push_back(button);
+	}
 }
 
 Display::~Display()
 {
-	resetButtons();
+	while (!m_buttons.empty()) { delete *m_buttons.begin(); m_buttons.erase(m_buttons.begin()); }
 }
 
 void Display::setEstate(Estate *estate)
@@ -68,6 +91,19 @@ void Display::setText(const std::string text)
 	m_text = text;
 }
 
+void Display::setText(const char *fmt, ...)
+{
+	va_list arg;
+	char buf[2048];
+
+	va_start(arg, fmt);
+	vsnprintf(buf, sizeof(buf), fmt, arg);
+	va_end(arg);
+	buf[sizeof(buf)-1] = 0;
+
+	m_text = std::string(buf);
+}
+
 const std::string Display::text()
 {
 	return escapeXML(m_text);
@@ -77,28 +113,21 @@ void Display::addButton(const std::string command, const std::string caption, co
 {
 	DisplayButton *button = new DisplayButton(command, caption, enabled);
 	m_buttons.push_back(button);
-	m_clearButtons = false;
 }
 
 void Display::resetButtons()
 {
-	while (!m_buttons.empty()) { delete *m_buttons.begin(); m_buttons.erase(m_buttons.begin()); }
 	m_clearButtons = true;
 }
 
-void Display::setClearText(bool clearText)
+void Display::resetText()
 {
-	m_clearText = clearText;
+	m_clearText = true;
 }
 
-bool Display::clearText()
+void Display::resetEstate()
 {
-	return m_clearText;
-}
-
-void Display::setClearButtons(bool clearButtons)
-{
-	m_clearButtons = clearButtons;
+	m_clearEstate = true;
 }
 
 bool Display::clearButtons()
@@ -106,6 +135,16 @@ bool Display::clearButtons()
 	return m_clearButtons;
 }
 
+bool Display::clearText()
+{
+	return m_clearText;
+}
+
+bool Display::clearEstate()
+{
+	return m_clearEstate;
+}
+
 std::vector<DisplayButton *> Display::buttons()
 {
 	return m_buttons;
