@@ -43,8 +43,7 @@
 #include "display.h"
 
 Game::Game(int id)
- :	GameObject(id, GGame),
- 	m_pWinner( 0 )
+ :	GameObject(id, GGame)
 {
 	m_status = Config;
 
@@ -553,7 +552,7 @@ void Game::start(Player *pInput)
 
 void Game::setTokenLocation(Player *pInput, unsigned int estateId)
 {
-	if (!clientsMoving() || !pInput || !m_pTurn)
+	if (!clientsMoving())
 		return;
 
 	Estate *estateLoc = findEstate(estateId);
@@ -561,54 +560,48 @@ void Game::setTokenLocation(Player *pInput, unsigned int estateId)
 	if ( !estateLoc || !pInput->tokenLocation() || !m_pTurn->destination() || (pInput->tokenLocation() == estateLoc) )
 		return;
 
-	bool useNext = false;
-	unsigned int money = 0;
-
 //	printf("Game::setTokenLocation, P:%d PTL:%d EID:%d\n", pInput->id(), pInput->tokenLocation()->id(), estateId);
 
-	Estate *estate = 0;
-	for (std::vector<Estate *>::iterator it = m_estates.begin() ;;)
-	{
-		if (it == m_estates.end())
-		{
+	std::vector<Estate *>::iterator it;
+	// find current player estate
+	for (it = m_estates.begin(); it != m_estates.end(); it++) {
+		if (*it == pInput->tokenLocation()) {
+//			printf("Game::setTokenLocation, %d==PTL\n", (*it)->id());
+			it++;
+			break;
+		}
+	}
+
+	unsigned int money = 0;
+	while (1) {
+		if (it == m_estates.end()) {
 //			printf("Game::setTokenLocation, reloop\n");
 			it = m_estates.begin();
-			continue;
 		}
-		if (!(estate = *it))
+		Estate *estate = *it;
+
+		if (estate->getIntProperty("passmoney")) {
+//			printf("Game::setTokenLocation, pass:%d\n", estate->id());
+			Display display;
+			display.setText("%s passes %s and gets %d.", m_pTurn->getStringProperty("name").c_str(), estate->getStringProperty("name").c_str(), estate->getIntProperty("passmoney"));
+			pInput->sendDisplayMsg(&display);
+			money += estate->getIntProperty("passmoney");
+			pInput->ioWrite("<monopd><playerupdate playerid=\"%d\" money=\"%d\"/></monopd>\n", m_pTurn->id(), m_pTurn->getIntProperty("money") + money);
+		}
+
+		if (estate == m_pTurn->destination()) {
+//			printf("Game::setTokenLocation, setPTL:0\n");
+			pInput->setTokenLocation(0); // Player is done moving
 			break;
-
-		if (useNext)
-		{
-			if (estate->getIntProperty("passmoney"))
-			{
-//				printf("Game::setTokenLocation, pass:%d\n", estate->id());
-				Display display;
-				display.setText("%s passes %s and gets %d.", m_pTurn->getStringProperty("name").c_str(), estate->getStringProperty("name").c_str(), estate->getIntProperty("passmoney"));
-				pInput->sendDisplayMsg(&display);
-				money += estate->getIntProperty("passmoney");
-				pInput->ioWrite("<monopd><playerupdate playerid=\"%d\" money=\"%d\"/></monopd>\n", m_pTurn->id(), m_pTurn->getIntProperty("money") + money);
-			}
-			if (estate == m_pTurn->destination())
-			{
-//				printf("Game::setTokenLocation, setPTL:0\n");
-				pInput->setTokenLocation(0); // Player is done moving
-				break;
-			}
-			if (estate == estateLoc)
-			{
-//				printf("Game::setTokenLocation, setPTL:%d\n", estateId);
-				pInput->setTokenLocation(estate); // Player is not done moving
-				break;
-			}
-		}
-		else if (estate == pInput->tokenLocation())
-		{
-//			printf("Game::setTokenLocation, useNext:%d==PTL\n", estate->id());
-			useNext = true;
 		}
 
-		++it;
+		if (estate == estateLoc) {
+//			printf("Game::setTokenLocation, setPTL:%d\n", estateId);
+			pInput->setTokenLocation(estate); // Player is not done moving
+			break;
+		}
+
+		it++;
 	}
 
 	// Find out if there are still clients busy with movement.
@@ -616,38 +609,33 @@ void Game::setTokenLocation(Player *pInput, unsigned int estateId)
 		return;
 
 	// Land player!
-	bool endTurn = landPlayer(m_pTurn, false);
-
-	if (clientsMoving())
-	{
-		// Don't do a thing.. just wait until they are done.
-	}
-	else if (m_pTurn->getBoolProperty("can_buyestate"))
-	{
-		// Don't do a thing..
-	}
-	else if (endTurn)
-		m_pTurn->endTurn();
+	landPlayer(m_pTurn, false);
+	// game might be ended if a player is bankrupt after landing (eg. could not pay rent), thus m_pTurn can be NULL here
+	if (m_pTurn) m_pTurn->endTurn();
 }
 
 void Game::tokenMovementTimeout()
 {
+	// game might be ended if a player left while moving, thus m_pTurn can be NULL here
+	if (!m_pTurn) {
+		return;
+	}
+
+	if (!m_pTurn->destination()) {
+		return;
+	}
+
 	// Mark all clients as non moving.
-	Player *pTmp = 0;
-	for(std::vector<Player *>::iterator it = m_players.begin(); it != m_players.end() && (pTmp = *it) ; ++it)
-		if (pTmp->tokenLocation())
-		{
-			if (m_pTurn && m_pTurn->destination())
-			{
-				m_pTurn->setProperty("location", m_pTurn->destination()->id(), this);
-				m_pTurn->setBoolProperty("directmove", true, this);
-				setTokenLocation(pTmp, m_pTurn->destination()->id());
-			}
+	for(std::vector<Player *>::iterator it = m_players.begin(); it != m_players.end(); ++it) {
+		if ((*it)->tokenLocation()) {
+			setTokenLocation(*it, m_pTurn->destination()->id());
 		}
+	}
 }
 
 unsigned int Game::auctionTimeout()
 {
+	// game might be ended while an auction was in progress (eg. a player left), thus m_auction can be NULL here
 	if (!m_auction)
 		return 0;
 
@@ -672,6 +660,7 @@ unsigned int Game::auctionTimeout()
 		}
 
 		completeAuction();
+		m_pTurn->endTurn();
 		return 0;
 	}
 
@@ -747,6 +736,11 @@ void Game::delDebt(Debt *debt)
 			m_debts.erase(it);
 			break;
 		}
+
+	if (debt == m_auctionDebt) {
+		m_auctionDebt = 0;
+	}
+
 	Player *pFrom = debt->from();
 	if ( !findDebt(pFrom) )
 		pFrom->setBoolProperty("hasdebt", false);
@@ -761,8 +755,6 @@ void Game::solveDebts(Player *pInput, const bool &verbose)
 	{
 		if (verbose)
 			ioError("You don't have any debts to pay off!");
-		if (!clientsMoving() && !m_auction && !m_pausedForDialog)
-			pInput->endTurn();
 		return;
 	}
 
@@ -784,11 +776,8 @@ void Game::solveDebts(Player *pInput, const bool &verbose)
 		display.resetButtons(); /* Remove declare bankruptcy button */
 		pInput->sendDisplayMsg(&display);
 
-		completeAuction();
+		m_pTurn->endTurn();
 	}
-
-	if (!clientsMoving() && !m_auction && !m_pausedForDialog)
-		pInput->endTurn();
 }
 
 bool Game::solveDebt( Debt *debt )
@@ -808,10 +797,8 @@ bool Game::solveDebt( Debt *debt )
 	Display display;
 	display.setText("%s pays off a %d debt to %s.", pFrom->getStringProperty("name").c_str(), debt->amount(), (pCreditor ? pCreditor->getStringProperty("name").c_str() : "Bank"));
 	sendDisplayMsg(&display);
-	if (debt == m_auctionDebt) {
-		m_auctionDebt = 0;
-	}
 	delDebt(debt);
+	completeAuction();
 	return true;
 }
 
@@ -1039,13 +1026,8 @@ void Game::acceptTrade(Player *pInput, char *data)
 
 	trade->setPlayerAccept(pInput, true);
 
-	if (trade->allAccept())
-	{
-		// Complete trade
+	if (trade->allAccept()) {
 		completeTrade(trade);
-		delTrade(trade);
-		if (!clientsMoving() && !m_auction && !m_pausedForDialog)
-			m_pTurn->endTurn();
 	}
 }
 
@@ -1091,7 +1073,7 @@ void Game::completeTrade(Trade *trade)
 			delete object; // was temporarily created to serve as trade object
 			if (!pFrom->payMoney(money))
 			{
-				display2.setText("%s owes %d to %s. Game paused, %s is not solvent. Player needs to raise %d in cash first.", pFrom->getStringProperty("name").c_str(), money, pTo->getStringProperty("name").c_str(), pFrom->getStringProperty("name").c_str(), (money - pFrom->getIntProperty("money")));
+				display2.setText("Game paused, %s owes %d to %s but is not solvent. Player needs to raise %d in cash first.", pFrom->getStringProperty("name").c_str(), money, pTo->getStringProperty("name").c_str(), (money - pFrom->getIntProperty("money")));
 				sendDisplayMsg(&display2);
 
 				newDebt(pFrom, pTo, 0, money);
@@ -1115,6 +1097,8 @@ void Game::completeTrade(Trade *trade)
 	for(std::vector<Player *>::iterator it = m_players.begin(); it != m_players.end() && (player = *it) ; ++it)
 		if (trade->hasPlayer(player))
 			solveDebts(player);
+
+	delTrade(trade);
 }
 
 void Game::abortAuction()
@@ -1124,12 +1108,12 @@ void Game::abortAuction()
 
 	printf("Game::abortAuction()\n");
 
-	ioWrite("<monopd><auctionupdate auctionid=\"%d\" status=\"%d\"/></monopd>\n", m_auction->id(), Auction::Sold);
+	if (m_auction->status() != Auction::Sold) {
+		ioWrite("<monopd><auctionupdate auctionid=\"%d\" status=\"%d\"/></monopd>\n", m_auction->id(), Auction::Sold);
+	}
 
 	delete m_auction;
 	m_auction = NULL;
-
-	m_pTurn->endTurn();
 }
 
 void Game::completeAuction()
@@ -1149,8 +1133,6 @@ void Game::completeAuction()
 
 	delete m_auction;
 	m_auction = NULL;
-
-	m_pTurn->endTurn();
 }
 
 void Game::sendDisplayMsg(Display *display, Player *except) {
@@ -1226,46 +1208,54 @@ Estate *Game::findEstate(int id)
 
 Estate *Game::findNextEstate(EstateGroup *group, Estate *startEstate)
 {
-	bool useNext = true;
-	if (startEstate)
-		useNext = false;
-
-	Estate *eTmp = 0, *eFirst = 0;
-	for (std::vector<Estate *>::iterator eIt = m_estates.begin() ; eIt != m_estates.end() && (eTmp = *eIt) ; eIt++)
-	{
-		if (startEstate && startEstate == eTmp)
-			useNext = true;
-		if (eTmp->group() == group)
-		{
-			if (useNext==true)
-				return eTmp;
-			else if (!eFirst)
-				eFirst = eTmp;
+	std::vector<Estate *>::iterator it = m_estates.begin();
+	if (startEstate) {
+		for (; it != m_estates.end(); it++) {
+			if (startEstate == *it) {
+				break;
+			}
 		}
 	}
-	return eFirst;
+	for (; it != m_estates.end(); it++) {
+		if ((*it)->group() == group) {
+			return *it;
+		}
+	}
+	// reloop
+	if (startEstate) {
+		for (it = m_estates.begin(); it != m_estates.end(); it++) {
+			if ((*it)->group() == group) {
+				return *it;
+			}
+		}
+	}
+	return NULL;
 }
 
 Estate *Game::findNextJailEstate(Estate *startEstate)
 {
-	bool useNext = true;
-	if (startEstate)
-		useNext = false;
-
-	Estate *eTmp = 0, *eFirst = 0;
-	for (std::vector<Estate *>::iterator eIt = m_estates.begin() ; eIt != m_estates.end() && (eTmp = *eIt) ; eIt++)
-	{
-		if (startEstate && startEstate == eTmp)
-			useNext = true;
-		if (eTmp->getBoolProperty("jail"))
-		{
-			if (useNext==true)
-				return eTmp;
-			else if (!eFirst)
-				eFirst = eTmp;
+	std::vector<Estate *>::iterator it = m_estates.begin();
+	if (startEstate) {
+		for (; it != m_estates.end(); it++) {
+			if (startEstate == *it) {
+				break;
+			}
 		}
 	}
-	return eFirst;
+	for (; it != m_estates.end(); it++) {
+		if ((*it)->getBoolProperty("jail")) {
+			return *it;
+		}
+	}
+	// reloop
+	if (startEstate) {
+		for (it = m_estates.begin(); it != m_estates.end(); it++) {
+			if ((*it)->getBoolProperty("jail")) {
+				return *it;
+			}
+		}
+	}
+	return NULL;
 }
 
 void Game::transferEstate(Estate *estate, Player *player, const bool verbose)
@@ -1440,10 +1430,13 @@ Player *Game::findPlayer(int playerId)
 
 void Game::removePlayer(Player *p)
 {
-	if (p->getBoolProperty("hasturn")) {
-		if (clientsMoving()) {
-			tokenMovementTimeout();
-		}
+	// note: m_pTurn is NULL when a game is finished
+	if (p == m_pTurn) {
+		// cancel any tax dialog
+		m_pausedForDialog = false;
+		// player left game during its turn, cancel any token movement
+		setAllClientsMoving(0);
+		// and update turn -before- removing player from players vector
 		updateTurn();
 	}
 
@@ -1593,32 +1586,30 @@ unsigned int Game::playerAssets(Player *player)
 
 void Game::updateTurn()
 {
-	if (m_status == Game::End)
-		return;
-
-	Player *pOldTurn = m_pTurn;
 	// Disable turn, roll and buy.
-	pOldTurn->setTurn(false);
+	m_pTurn->setTurn(false);
 
-	Player *player = 0, *pFirst = 0;
-	bool useNext = false;
-	for(std::vector<Player *>::iterator it = m_players.begin() ; it != m_players.end() && (player = *it) ; ++it)
-	{
-		if (!pFirst && !player->getBoolProperty("bankrupt") && !player->getBoolProperty("spectator"))
-			pFirst = player;
-
-		if (player == pOldTurn)
-			useNext = true;
-		else if (useNext && !player->getBoolProperty("bankrupt") && !player->getBoolProperty("spectator"))
-		{
-			m_pTurn = player;
-			useNext = false;
+	std::vector<Player *>::iterator it;
+	// find current player
+	for(it = m_players.begin(); it != m_players.end(); it++) {
+		if (*it == m_pTurn) {
+			it++;
 			break;
 		}
 	}
-	if (useNext && pFirst) {
-		m_pTurn = pFirst;
-		setProperty("turn", getIntProperty("turn") +1);
+
+	// find next player
+	while (1) {
+		if (it == m_players.end()) {
+			it = m_players.begin();
+			setProperty("turn", getIntProperty("turn") +1);
+		}
+		Player *player = *it;
+		if (!player->getBoolProperty("bankrupt") && !player->getBoolProperty("spectator")) {
+			m_pTurn = player;
+			break;
+		}
+		it++;
 	}
 
 	// Set turn.
@@ -1629,46 +1620,43 @@ void Game::updateTurn()
 	m_pTurn->setTurn(true);
 }
 
-bool Game::landPlayer(Player *pTurn, const bool directMove, const std::string &rentMath)
+void Game::landPlayer(Player *pTurn, const bool directMove, const std::string &rentMath)
 {
 	Estate *destination = pTurn->destination();
+	if (destination) {
+		std::vector<Estate *>::iterator it;
 
-	if (destination)
-	{
-		bool useNext = false;
-		unsigned int money = 0;
-		Estate *estate = 0;
-		for (std::vector<Estate *>::iterator it = m_estates.begin() ;;)
-		{
-			if (it == m_estates.end())
-			{
-				it = m_estates.begin();
-				continue;
-			}
-
-			if (!(estate = *it))
+		// find current player estate
+		for (it = m_estates.begin(); it != m_estates.end(); it++) {
+			if (*it == pTurn->estate()) {
+				it++;
 				break;
-
-			if (useNext)
-			{
-				if (estate->getIntProperty("passmoney"))
-				{
-					money += estate->getIntProperty("passmoney");
-					// Write incremental message for direct moves, token
-					// confirmation or timeout didn't do it yet.
-					if (directMove) {
-						Display display;
-						display.setText("%s passes %s and gets %d.", pTurn->getStringProperty("name").c_str(), estate->getStringProperty("name").c_str(), estate->getIntProperty("passmoney"));
-						sendDisplayMsg(&display);
-					}
-				}
-				if (estate == destination)
-					break;
 			}
-			else if (estate == pTurn->estate())
-				useNext = true;
+		}
 
-			++it;
+		unsigned int money = 0;
+		while (1) {
+			if (it == m_estates.end()) {
+				it = m_estates.begin();
+			}
+			Estate *estate = *it;
+
+			if (estate->getIntProperty("passmoney")) {
+				money += estate->getIntProperty("passmoney");
+				// Write incremental message for direct moves, token
+				// confirmation or timeout didn't do it yet.
+				if (directMove) {
+					Display display;
+					display.setText("%s passes %s and gets %d.", pTurn->getStringProperty("name").c_str(), estate->getStringProperty("name").c_str(), estate->getIntProperty("passmoney"));
+					sendDisplayMsg(&display);
+				}
+			}
+
+			if (estate == destination) {
+				break;
+			}
+
+			it++;
 		}
 		pTurn->setEstate(destination);
 		pTurn->setDestination(0);
@@ -1682,7 +1670,6 @@ bool Game::landPlayer(Player *pTurn, const bool directMove, const std::string &r
 	// sent earlier.
 
 	// What properties does the estate we are landing on have?
-	bool endTurn = true;
 	Player *pOwner = 0;
 	Estate *es = pTurn->estate();
 
@@ -1713,7 +1700,7 @@ bool Game::landPlayer(Player *pTurn, const bool directMove, const std::string &r
 			ioError("This gameboard does not have a jail estate.");
 
 		updateTurn();
-		return true;
+		return;
 	}
 
 	// Any estate can have a pot of gold. This is handled before all other
@@ -1745,7 +1732,7 @@ bool Game::landPlayer(Player *pTurn, const bool directMove, const std::string &r
 				sendDisplayMsg(&display);
 
 				newDebt(pTurn, 0, ePayTarget, estateMoney);
-				return false;
+				return;
 			}
 			else if (ePayTarget && getBoolConfigOption("collectfines"))
 				ePayTarget->addMoney(estateMoney);
@@ -1787,7 +1774,7 @@ bool Game::landPlayer(Player *pTurn, const bool directMove, const std::string &r
 
 				// TODO: port this into a blocking bool in Display which we can check, will be more generic
 				m_pausedForDialog = true;
-				return false;
+				return;
 			}
 		}
 		else
@@ -1805,7 +1792,7 @@ bool Game::landPlayer(Player *pTurn, const bool directMove, const std::string &r
 			sendDisplayMsg(&display);
 
 			newDebt(pTurn, 0, ePayTarget, payAmount);
-			return false;
+			return;
 		}
 		else if (ePayTarget && getBoolConfigOption("collectfines"))
 			ePayTarget->addMoney(payAmount);
@@ -1821,7 +1808,7 @@ bool Game::landPlayer(Player *pTurn, const bool directMove, const std::string &r
 		if (getBoolConfigOption("alwaysshuffle"))
 			cardGroup->shuffleCards();
 
-		endTurn = giveCard(pTurn, cardGroup->nextCard());
+		giveCard(pTurn, cardGroup->nextCard());
 	}
 
 	// Calculate rent for owned estates or offer them for sale.
@@ -1854,7 +1841,7 @@ bool Game::landPlayer(Player *pTurn, const bool directMove, const std::string &r
 					sendDisplayMsg(&display);
 
 					newDebt(pTurn, pOwner, 0, payAmount);
-					return false;
+					return;
 				}
 				Display display;
 				display.setText("%s pays %d rent to %s.", pTurn->getStringProperty("name").c_str(), payAmount, pOwner->getStringProperty("name").c_str());
@@ -1877,17 +1864,17 @@ bool Game::landPlayer(Player *pTurn, const bool directMove, const std::string &r
 			display.addButton(".ea", "Auction", (getBoolConfigOption("auctionsenabled") && totalAssets()));
 			display.addButton(".E", "End Turn", !getBoolConfigOption("auctionsenabled"));
 			pTurn->sendDisplayMsg(&display);
-			return false;
+			return;
 		}
 	}
-	return endTurn;
 }
-bool Game::giveCard(Player *pTurn, Card *card)
+
+void Game::giveCard(Player *pTurn, Card *card)
 {
 	if (!card)
 	{
 		ioError(pTurn->name() + " should get a card, but there don't seem to be any available!");
-		return true;
+		return;
 	}
 
 	Display display;
@@ -1922,7 +1909,6 @@ bool Game::giveCard(Player *pTurn, Card *card)
 				sendDisplayMsg(&display);
 
 				newDebt(pTurn, 0, ePayTarget, payAmount);
-				return false;
 			}
 			else if (ePayTarget && getBoolConfigOption("collectfines"))
 				ePayTarget->addMoney(payAmount);
@@ -2000,7 +1986,6 @@ bool Game::giveCard(Player *pTurn, Card *card)
 				sendDisplayMsg(&display);
 
 				newDebt(pTurn, 0, ePayTarget, payAmount);
-				return false;
 			}
 			else if (ePayTarget && getBoolConfigOption("collectfines"))
 				ePayTarget->addMoney(payAmount);
@@ -2022,9 +2007,8 @@ bool Game::giveCard(Player *pTurn, Card *card)
 			else
 				ioError("Could not find next estate on gameboard.");
 		}
-		return landPlayer(pTurn, true, card->rentMath());
+		landPlayer(pTurn, true, card->rentMath());
 	}
-	return true;
 }
 
 void Game::declareBankrupt(Player *pInput)
@@ -2054,7 +2038,7 @@ void Game::bankruptPlayer(Player *pBroke)
 		++it;
 	}
 
-	// highest bidder might be the player who just left
+	// highest bidder declared bankruptcy (or left)
 	if (m_auction && m_auction->highBidder() == pBroke) {
 		abortAuction();
 	}
@@ -2081,55 +2065,58 @@ void Game::bankruptPlayer(Player *pBroke)
 
 	// Player might still have assets, when he goes bankrupt without debts
 	// (disconnection timeout), so we'll enforce giving assets to the Bank.
-	if (pBroke->assets())
-		enforceDebt(pBroke);
+	enforceDebt(pBroke);
 
 	// Count active (non-bankrupt) players
 	int activePlayers = 0;
 	Player *pTmp = 0;
+	Player *pWinner = 0;
 	for(std::vector<Player *>::iterator it = m_players.begin(); it != m_players.end() && (pTmp =  *it) ; ++it)
 		if (pTmp && !pTmp->getBoolProperty("bankrupt") && !pTmp->getBoolProperty("spectator"))
 		{
 			activePlayers++;
-			m_pWinner = pTmp;
+			pWinner = pTmp;
 		}
 
-	if (activePlayers == 1)
-	{
+	if (activePlayers == 1) {
 		// a user who disconnect might end the game while an auction was running
 		abortAuction();
+		// game might be ended if a player left while a player was moving
+		setAllClientsMoving(0);
 
 		m_status = End;
 		setProperty("status", "end");
-		syslog(LOG_INFO, "game %d ended: %s wins with a fortune of %d.", m_id, m_pWinner->name().c_str(), m_pWinner->assets());
+		syslog(LOG_INFO, "game %d ended: %s wins with a fortune of %d.", m_id, pWinner->name().c_str(), pWinner->assets());
 		setBoolProperty("canbewatched", false);
 
 		Display display;
 		display.resetButtons();  /* Reset any button, game might end if a player left while we were asked to buy an estate for example */
-		display.setText("The game has ended! %s wins with a fortune of %d!", m_pWinner->getStringProperty("name").c_str(), m_pWinner->assets());
+		display.setText("The game has ended! %s wins with a fortune of %d!", pWinner->getStringProperty("name").c_str(), pWinner->assets());
 		display.addButton(".gx", "New Game", true);
 		sendDisplayMsg(&display);
+
+		m_pTurn->setTurn(false);
+		m_pTurn = NULL;
+		return;
 	}
-	else
-	{
-		m_pWinner = 0;
 
-		if (activePlayers && !m_debts.size())
-		{
-			if (debtsWereSolved) {
-				Display display;
-				display.setText("All debts are settled, game continues.");
-				sendDisplayMsg(&display);
-			}
-
-			// Update turn. Always when this function is called as command
-			// and possibly necessary when this function is called for a
-			// disconnected user.
-
-			if (pBroke == m_pTurn)
-				updateTurn();
-		}
+	if (m_debts.size()) {
+		return;
 	}
+
+	if (debtsWereSolved) {
+		Display display;
+		display.setText("All debts are settled, game continues.");
+		sendDisplayMsg(&display);
+	}
+
+	if (pBroke == m_pTurn) {
+		updateTurn();
+		return;
+	}
+
+	// necessary when an auction bidder cannot pay and then declared bankruptcy
+	m_pTurn->endTurn();
 }
 
 int Game::totalAssets()
@@ -2172,19 +2159,21 @@ int Game::debts()
 unsigned int Game::clientsMoving()
 {
 	unsigned int moving = 0;
-	Player *pTmp = 0;
-	for(std::vector<Player *>::iterator it = m_players.begin(); it != m_players.end() && (pTmp = *it) ; ++it)
-		if (pTmp->tokenLocation())
+	for(std::vector<Player *>::iterator it = m_players.begin(); it != m_players.end(); ++it) {
+		if ((*it)->tokenLocation()) {
 			moving++;
+		}
+	}
 	return moving;
 }
 
 void Game::setAllClientsMoving(Estate *estate)
 {
-	Player *pTmp = 0;
-	for(std::vector<Player *>::iterator it = m_players.begin(); it != m_players.end() && (pTmp = *it) ; ++it)
-	if (pTmp->socket())
-		pTmp->setTokenLocation(estate);
+	for(std::vector<Player *>::iterator it = m_players.begin(); it != m_players.end(); ++it) {
+		if ((*it)->socket()) {
+			(*it)->setTokenLocation(estate);
+		}
+	}
 }
 
 void Game::sendStatus(Status status, Player *p)
