@@ -278,32 +278,54 @@ void Player::rollDice()
 
 void Player::endTurn(bool userRequest)
 {
-	if (m_game->status() == Game::End)
+	if (!getBoolProperty("hasturn")) {
+		if (userRequest)
+			ioError("You cannot end your turn, this is not your turn.");
 		return;
+	}
 
-	if (!getBoolProperty("hasturn"))
+	if (m_game->pausedForDialog()) {
+		if (userRequest)
+			ioError("You cannot end your turn, you must answer the dialog first.");
 		return;
+	}
 
-	if (m_game->findDebt(this))
-    {
-    	if (userRequest)
-    		ioError("You have a debt, cannot end turn.");
-    	return;
-    }
+	if (m_game->auction()) {
+		if (userRequest)
+			ioError("You cannot end your turn, an auction is in progress.");
+		return;
+	}
 
-	if (getBoolProperty("can_buyestate"))
-	{
-		if (m_game->getBoolConfigOption("auctionsenabled"))
-		{
-			if (userRequest)
-				ioError("You cannot end your turn, you must either buy or auction the property you are on.");
+	if (m_game->clientsMoving()) {
+		if (userRequest)
+			ioError("You cannot end your turn, other players are still moving.");
+		return;
+	}
+
+	if (m_game->debts()) {
+		if (userRequest)
+			ioError("You cannot end your turn, there are debts to be settled.");
+		return;
+	}
+
+	if (getBoolProperty("can_buyestate")) {
+		if (!userRequest) {
 			return;
 		}
-		else if (!userRequest)
+
+		if (m_game->getBoolConfigOption("auctionsenabled")) {
+			ioError("You cannot end your turn, you must either buy or auction the property you are on.");
 			return;
+		}
+
+		setBoolProperty("can_buyestate", false);
+		setBoolProperty("canauction", false);
+		Display display;
+		display.resetButtons(); /* Remove buy estate buttons: Buy, Auction, End Turn */
+		sendDisplayMsg(&display);
 	}
-	else if (getBoolProperty("canrollagain"))
-	{
+
+	if (getBoolProperty("canrollagain")) {
 		Display display;
 		display.setText("%s may roll again.", getStringProperty("name").c_str());
 		m_game->sendDisplayMsg(&display, this);
@@ -314,32 +336,19 @@ void Player::endTurn(bool userRequest)
 		setBoolProperty("canrollagain", false);
 		return;
 	}
-	else if (getBoolProperty("can_roll"))
-	{
+
+	if (getBoolProperty("can_roll")) {
 		if (userRequest)
 			ioError("You cannot end your turn, you must roll first!");
 		return;
 	}
-	else if (getBoolProperty("jailed"))
-	{
+
+	if (getBoolProperty("jailed")) {
 		if (userRequest)
 			ioError("You cannot end your turn while jailed.");
 		return;
 	}
 
-	else if (m_game->pausedForDialog())
-	{
-		if (userRequest)
-			ioError("You must answer the dialog first.");
-		return;
-	}
-
-	if (userRequest) {
-		Display display;
-		display.resetButtons(); /* Remove buy estate buttons: Buy, Auction, End Turn */
-		sendDisplayMsg(&display);
-	}
-
 	// Turn goes to next player
 	m_game->updateTurn();
 }
